classdef DBSCAN_map < handle 
    %UNTITLED Summary of this class goes here
    %   Detailed explanation goes here

    properties
        dataCooordinates_nm  =[]
        epsilon = []
        N = []
        clusterProperties = table()

        coordinateFieldNames = {'x_nm', 'y_nm'};

        parameterNames = {'epsilon', 'N'};

        propertyNames = {...
                'loc_number_AV', 'loc_number_SD', 'loc_number_MD',...
                'cluser_areas_AV', 'cluser_areas_SD', 'cluser_areas_MD',...
                'cluster_number'};
    end

    methods
        function obj = DBSCAN_map()
            %UNTITLED Construct an instance of this class
            %   Detailed explanation goes here

        end

        function [dataFileFullName] = dataImport(obj)
            %METHOD1 Summary of this method goes here
            %   Detailed explanation goes here
            
            [dataFileName, dataFilePath] = uigetfile(...
                {'*.csv','CSV files (*.csv)'; ...
                '*.*',  'All Files (*.*)'}, ...
                'Select a pointillistic data file for DBSCAN map creation');

            if dataFileName == 0
                return
            end

            dataFileFullName = fullfile(dataFilePath, dataFileName);

            originalData= readtable(dataFileFullName);

            coordinatesTable = obj.sliceData(originalData, obj.coordinateFieldNames);

            obj.dataCooordinates_nm = table2array(coordinatesTable);

            % delete the parameters/results of the previous analysis
            obj.epsilon = [];
            obj.N = [];
            obj.clusterProperties = table();

        end

        function createMap(obj, epsilon_vect, N_vect)

            if isempty(obj.dataCooordinates_nm)
                warning('There is no pointillistic data imported. Import one before calculation the DBSCAN map.')
                return
            end
            fprintf('Starting DBSCAN map generation with the following parameters\nradius [nm]:\t%s\nminPts:\t\t\t%s\n', num2str(epsilon_vect), num2str(N_vect))

            % enforce it to be integer type
            N_vect = uint16(N_vect);

            obj.N = N_vect;
            obj.epsilon = epsilon_vect;

            epsilon_number = numel(epsilon_vect);
            N_number = numel(N_vect);

            index_table = combinations(1:epsilon_number, 1:N_number);
            % alternative: combvec()
            index_table.Properties.VariableNames = obj.parameterNames;

            map_elementNumber.mean = zeros(N_number, epsilon_number);
            map_elementNumber.std = zeros(N_number, epsilon_number);

            clusterProperties_table = array2table(zeros(epsilon_number*N_number, 7));
            clusterProperties_table.Properties.VariableNames = obj.propertyNames;

            parfor_progress(epsilon_number*N_number);
            parfor idx = 1:(epsilon_number*N_number)
                radius = epsilon_vect(index_table.epsilon(idx));
                MinPts = N_vect(index_table.N(idx));
                
                [IDX, isnoise] = DBSCAN_boundaryIndexPrecalculation(obj.dataCooordinates_nm, radius, MinPts);

                [XC, nOfEl, volumeOfCl, distOfCl] = calculateClusterProperties(obj.dataCooordinates_nm, IDX);

                clusterProperties_table(idx, :) = {...
                    mean(nOfEl), std(nOfEl), median(nOfEl),...
                    mean(volumeOfCl), std(volumeOfCl), median(volumeOfCl),...
                    numel(nOfEl)};
                parfor_progress;
            end
            parfor_progress(0);
            

            obj.clusterProperties = clusterProperties_table;

            disp('Finished creating DBSCAN map.')
        end

        function visualize(obj)

            if isempty(obj.clusterProperties)
                warning('There is no cluster properties data to visualize. Run the DBSCAN map calculation first.')
                return
            end

            radius = obj.epsilon;
            MinPts = obj.N;

            loc_number_AV_map = obj.propertiesTableToMap('loc_number_AV');
            figureObject = visualizeMap(loc_number_AV_map, radius, MinPts, 'Average of Localization Number');
            figure(figureObject);

            loc_number_SD_map = obj.propertiesTableToMap('loc_number_SD');
            figureObject = visualizeMap(loc_number_SD_map, radius, MinPts, 'Standard Deviation of Localization Number');
            figure(figureObject);

            loc_number_MD_map = obj.propertiesTableToMap('loc_number_MD');
            figureObject = visualizeMap(loc_number_MD_map, radius, MinPts, 'Median of Localization Number');
            figure(figureObject);

            cluser_areas_AV_map = obj.propertiesTableToMap('cluser_areas_AV');
            figureObject = visualizeMap(cluser_areas_AV_map, radius, MinPts, 'Average of Cluster Areas');
            figure(figureObject);

            cluser_areas_SD_map = obj.propertiesTableToMap('cluser_areas_SD');
            figureObject = visualizeMap(cluser_areas_SD_map, radius, MinPts, 'Standard Deviation of Cluster Areas');
            figure(figureObject);

            cluser_areas_MD_map = obj.propertiesTableToMap('cluser_areas_MD');
            figureObject = visualizeMap(cluser_areas_MD_map, radius, MinPts, 'Median of Localization Number');
            figure(figureObject);

            cluster_number_map = obj.propertiesTableToMap('cluster_number');
            figureObject = visualizeMap(cluster_number_map, radius, MinPts, 'Number of Clusters');
            figure(figureObject);

        end

        function saveClusterProperties(obj)

            if isempty(obj.clusterProperties)
                warning('No cluster properties to save. Please run the DBSCAN map calculation first.')
                return
            end

            epsilon_N_table = combinations(obj.epsilon, obj.N);
            epsilon_N_table.Properties.VariableNames = obj.parameterNames;

            outputTable = [epsilon_N_table, obj.clusterProperties];

            [filename, pathname] = uiputfile(...
                {'*.csv','CSV files (*.csv)'; ...
                '*.*',  'All Files (*.*)'}, ...
                'Save the cluster properties into a tabular data file.');

            if filename == 0
                return
            end

            writetable(outputTable, fullfile(pathname, filename));
        end

        function loadClusterProperties(obj)

            [filename, pathname] = uigetfile(...
                {'*.csv','CSV files (*.csv)'; ...
                '*.*',  'All Files (*.*)'}, ...
                'Select a cluster properties table data file.');

            if filename == 0
                return
            end

            loadedTable = readtable(fullfile(pathname, filename));

            epsilon_N_table = obj.sliceData(loadedTable, obj.parameterNames);

            clusterProperties_table = obj.sliceData(loadedTable, obj.propertyNames);

            % delete the data/parameters/results of the previous analysis
            obj.dataCooordinates_nm = [];
            obj.epsilon = [];
            obj.N = [];
            obj.clusterProperties = table();


            % extract all occuring epsilon and N values in ascending order:
            radius = sort(unique(epsilon_N_table.(obj.parameterNames{1})));
            MinPts = sort(unique(epsilon_N_table.(obj.parameterNames{2})));

            if numel(radius)*numel(MinPts) ~= height(epsilon_N_table) 
                error(['The number of all possible combinations of the ...' ...
                    'loaded "Epsilon" (%s) and "N" (%s) must match the ...' ...
                    'number of rows (%s). Check the validity of the table'], numel(radius), numel(MinPts), height(epsilon_N_table));
            end

            obj.epsilon = radius;
            obj.N = MinPts;
            obj.clusterProperties = clusterProperties_table;
        end

    end

    methods (Access=private)

        function map = propertiesTableToMap(obj, table_field)
            map = reshape(obj.clusterProperties.(table_field), [numel(obj.N), numel(obj.epsilon)]);
        end
    end

    methods (Static, Access=private)

        function slicedData = sliceData(tabularData, chosenFieldNames)

            coordinateNames_ismember = ismember(chosenFieldNames, tabularData.Properties.VariableNames);
            missingCoordinates_cell = chosenFieldNames(~coordinateNames_ismember);
            missingCoordinates_text = sprintf('%s ; ',missingCoordinates_cell{:});
            if prod(coordinateNames_ismember) == 0
                error('The loaded cluster properties table missing some property fields: %s.', missingCoordinates_text);
            end

            coordianteNameIndices = ismember(tabularData.Properties.VariableNames, chosenFieldNames);
            slicedData = tabularData(:, coordianteNameIndices);

        end

    end

end